SystemOrganization addCategory: #'Zinc-SSO-OAuth2-Core'!

Object subclass: #ZnOAuth2ConsumerData
	instanceVariableNames: 'key secret redirectUrl'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-SSO-OAuth2-Core'!

!ZnOAuth2ConsumerData commentStamp: '<historical>' prior: 0!
I am ZnOAuth2ConsumerData holding the data necessary to ask an OAuth2 provider to do authentication on our behalf.

key
	- the application key

redirectUrl
	- The url to which to provider will redirect to

secret
	- the application secret!

!ZnOAuth2ConsumerData class methodsFor: 'instance creation' stamp: 'JanVanDeSandt 10/7/2012 16:04'!
key: aKey secret: aSecret

	^ self new setKey: aKey andSecret: aSecret! !

!ZnOAuth2ConsumerData class methodsFor: 'instance creation' stamp: 'JanVanDeSandt 12/7/2012 15:40'!
key: aKey secret: aSecret redirectUrl: anUrl

	^ self new setKey: aKey andSecret: aSecret andRedirectUrl: anUrl! !

!ZnOAuth2ConsumerData methodsFor: 'accessing' stamp: 'JanVanDeSandt 10/7/2012 13:37'!
key
	^ key! !

!ZnOAuth2ConsumerData methodsFor: 'accessing' stamp: 'JanVanDeSandt 12/7/2012 15:23'!
redirectUrl

	^ redirectUrl ! !

!ZnOAuth2ConsumerData methodsFor: 'accessing' stamp: 'JanVanDeSandt 10/7/2012 13:37'!
secret
	^ secret! !

!ZnOAuth2ConsumerData methodsFor: 'initialize-release' stamp: 'JanVanDeSandt 10/7/2012 16:03'!
setKey: aKey andSecret: aSecret

	key := aKey.
	secret := aSecret ! !

!ZnOAuth2ConsumerData methodsFor: 'initialize-release' stamp: 'JanVanDeSandt 12/7/2012 15:40'!
setKey: aKey andSecret: aSecret andRedirectUrl: anUrl

	self setKey: aKey andSecret: aSecret.
	redirectUrl := anUrl! !

Object subclass: #ZnOAuth2Session
	instanceVariableNames: 'consumerData redirectUrl scopes accessToken'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-SSO-OAuth2-Core'!

!ZnOAuth2Session commentStamp: '<historical>' prior: 0!
I am ZnOAuth2Session, representing server side state tracking an OAuth2 session.

consumerData  
	- needed to setup an authentication request with a provider
	
accessToken
	- the result of a successful authentication
	
scopes
	- collection of strings representing the permissions 
	
redirectUrl
	- the URL needed by the provider to get back to us
!

ZnOAuth2Session subclass: #ZnFacebookOAuth2Session
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-SSO-OAuth2-Core'!

!ZnFacebookOAuth2Session commentStamp: '<historical>' prior: 0!
I am ZnFacebookOAuth2Session.
I am a ZnOAuth2Session.!

!ZnFacebookOAuth2Session class methodsFor: 'resources' stamp: 'JanVanDeSandt 12/23/2012 23:10'!
scopeEmail
	"Provides access to the user's primary email address in the email property.
	See: https://developers.facebook.com/docs/reference/login/email-permissions/ "
	
	^ 'email'! !

!ZnFacebookOAuth2Session class methodsFor: 'resources' stamp: 'JanVanDeSandt 12/23/2012 23:14'!
scopeUserLikes
	"Provides access to the list of all of the pages the user has liked as the likes connection
	See: https://developers.facebook.com/docs/reference/login/extended-profile-properties/"
	
	^ 'user_likes'! !

!ZnFacebookOAuth2Session class methodsFor: 'resources' stamp: 'JanVanDeSandt 12/23/2012 23:16'!
scopeUserPhotos
	"Provides access to the photos the user has uploaded.
	See: https://developers.facebook.com/docs/reference/login/extended-profile-properties/"
	
	^ 'user_photos'! !

!ZnFacebookOAuth2Session class methodsFor: 'resources' stamp: 'JanVanDeSandt 12/23/2012 23:26'!
ssoScopes
	"Answer a collection of scopes that can be used for Authentication. "

	^ Array with: self scopeEmail! !

!ZnFacebookOAuth2Session methodsFor: 'public' stamp: 'JanVanDeSandt 12/23/2012 23:34'!
authenticationUrl

	^ (ZnUrl fromString: 'https://graph.facebook.com/oauth/authorize')
		queryAt: 'client_id' put: self consumerData key ;
		queryAt: 'redirect_uri' put: self redirectUrl ;
		queryAt: 'scope' put: self scopeParameter ;
		yourself.
! !

!ZnFacebookOAuth2Session methodsFor: 'public' stamp: 'JanVanDeSandt 10/7/2012 14:31'!
getUserData

	| url response |
	
	url := (ZnUrl fromString: 'https://graph.facebook.com/me')
		queryAt: 'access_token' put: self accessToken;
		yourself.

	response := ZnClient new get: url.
	
	^ NeoJSONReader fromString: response! !

!ZnFacebookOAuth2Session methodsFor: 'public' stamp: 'JanVanDeSandt 12/7/2012 15:37'!
handleAuthenticationCallback: aParameterDictionary 

	| code url oauthResponse |
	
	code := aParameterDictionary at: 'code' ifAbsent: [ ^nil ].

	url := (ZnUrl fromString: 'https://graph.facebook.com/oauth/access_token')
		queryAt: 'client_id' put: self consumerData key ;
		queryAt: 'redirect_uri' put: self redirectUrl ;
		queryAt: 'client_secret' put: self consumerData secret ;
		queryAt: 'code' put: code;
		yourself.
	
	oauthResponse := ZnClient new get: url.
	
	accessToken := (self fieldDictionaryFrom: oauthResponse) at: 'access_token'.! !

ZnOAuth2Session subclass: #ZnGoogleOAuth2Session
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-SSO-OAuth2-Core'!

!ZnGoogleOAuth2Session commentStamp: '<historical>' prior: 0!
I am ZnGoogleOAuth2Session.
I am a ZnOAuthSession.

Google OAuth2 docs: https://developers.google.com/accounts/docs/OAuth2WebServer!

!ZnGoogleOAuth2Session class methodsFor: 'resources' stamp: 'JanVanDeSandt 12/23/2012 23:01'!
scopeUserinfoEmail
	"Gain read-only access to the user's email address."

	^ 'https://www.googleapis.com/auth/userinfo.email'! !

!ZnGoogleOAuth2Session class methodsFor: 'resources' stamp: 'JanVanDeSandt 12/23/2012 23:00'!
scopeUserinfoProfile
	"Gain read-only access to basic profile information, including a user identifier, name, profile photo, 
	profile URL, country, language, timezone, and birthdate. "

	^ 'https://www.googleapis.com/auth/userinfo.profile'! !

!ZnGoogleOAuth2Session class methodsFor: 'resources' stamp: 'JanVanDeSandt 12/23/2012 23:25'!
ssoScopes
	"Answer a collection of scopes that can be used for Authentication. "

	^ Array with: self scopeUserinfoProfile with: self scopeUserinfoEmail! !

!ZnGoogleOAuth2Session methodsFor: 'public' stamp: 'JanVanDeSandt 12/7/2012 16:11'!
authenticationUrl

	^self authenticationUrlWithState: 'no-op'! !

!ZnGoogleOAuth2Session methodsFor: 'public' stamp: 'JanVanDeSandt 12/23/2012 23:35'!
authenticationUrlWithState: aStateString

	| authUrl |

	authUrl := (ZnUrl fromString: 'https://accounts.google.com/o/oauth2/auth')
		queryAt: 'response_type' put: 'code' ;
		queryAt: 'client_id' put: self consumerData key ;
		queryAt: 'redirect_uri' put: self redirectUrl ;
		queryAt: 'scope' put: self scopeParameter ;
		queryAt: 'state' put: aStateString ;
		queryAt: 'access_type' put: 'online' ;
		yourself.
		
	^authUrl! !

!ZnGoogleOAuth2Session methodsFor: 'public' stamp: 'JanVanDeSandt 10/7/2012 15:27'!
getUserData

	| url response |
	
	url := (ZnUrl fromString: 'https://www.googleapis.com/oauth2/v1/userinfo')
		queryAt: 'access_token' put: self accessToken;
		yourself.

	response := ZnClient new get: url.
	
	^ NeoJSONReader fromString: response! !

!ZnGoogleOAuth2Session methodsFor: 'public' stamp: 'JanVanDeSandt 12/7/2012 16:58'!
handleAuthenticationCallback: aParameterDictionary 

	| code tokenRequest tokenResponse tokenData |
	
	code := aParameterDictionary at: 'code' ifAbsent: [ ^nil ].

	tokenRequest  := (ZnRequest post: (ZnUrl fromString: 'https://accounts.google.com/o/oauth2/token'))
		entity: (ZnApplicationFormUrlEncodedEntity new
			at: 'code' put: code ;
			at: 'client_id' put: self consumerData key ;
			at: 'client_secret' put: self consumerData secret ;
			at: 'redirect_uri' put: self redirectUrl ;
			at: 'grant_type' put: 'authorization_code' ;
			yourself) ;
		yourself.

	tokenResponse := ZnClient new 
		request: tokenRequest ;
		post.
	tokenData := NeoJSONReader fromString: tokenResponse.
		
	accessToken := tokenData at: 'access_token'.! !

ZnOAuth2Session subclass: #ZnMicrosoftOAuth2Session
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-SSO-OAuth2-Core'!

!ZnMicrosoftOAuth2Session commentStamp: '<historical>' prior: 0!
I am ZnMicrosoftOAuth2Session.
I am a ZnOAuth2Session.

Also see: http://msdn.microsoft.com/en-us/library/live/hh243647.aspx!

!ZnMicrosoftOAuth2Session class methodsFor: 'resources' stamp: 'JanVanDeSandt 12/23/2012 23:18'!
scopeBasic
	"Read access to a user's basic profile info. Also enables read access to a user's list of contacts.
	See: http://msdn.microsoft.com/en-us/library/live/hh243646.aspx "

	^ 'wl.basic'! !

!ZnMicrosoftOAuth2Session class methodsFor: 'resources' stamp: 'JanVanDeSandt 12/23/2012 23:21'!
scopeEmails
	"Read access to a user's personal, preferred, and business email addresses.
	See: http://msdn.microsoft.com/en-us/library/live/hh243646.aspx "

	^ 'wl.emails'! !

!ZnMicrosoftOAuth2Session class methodsFor: 'resources' stamp: 'JanVanDeSandt 12/23/2012 23:20'!
scopeSignin
	"Single sign-in behavior. With single sign-in, users who are already signed in to Live Connect
	are also signed in to your website.
	See: http://msdn.microsoft.com/en-us/library/live/hh243646.aspx "

	^ 'wl.signin'! !

!ZnMicrosoftOAuth2Session class methodsFor: 'resources' stamp: 'JanVanDeSandt 12/23/2012 23:25'!
ssoScopes
	"Answer a collection of scopes that can be used for Authentication. "

	^ Array with: self scopeSignin with: self scopeBasic with: self scopeEmails! !

!ZnMicrosoftOAuth2Session methodsFor: 'public' stamp: 'JanVanDeSandt 12/23/2012 23:36'!
authenticationUrl

	^ (ZnUrl fromString: 'https://login.live.com/oauth20_authorize.srf')
		queryAt: 'response_type' put: 'code' ;
		queryAt: 'client_id' put: self consumerData key ;
		queryAt: 'redirect_uri' put: self redirectUrl ;
		queryAt: 'scope' put: self scopeParameter ;
		yourself.
! !

!ZnMicrosoftOAuth2Session methodsFor: 'public' stamp: 'JanVanDeSandt 12/7/2012 21:06'!
getUserData

	| url response |
	
	url := (ZnUrl fromString: 'https://apis.live.net/v5.0/me')
		queryAt: 'access_token' put: self accessToken;
		yourself.

	response := ZnClient new get: url.
	
	^ NeoJSONReader fromString: response! !

!ZnMicrosoftOAuth2Session methodsFor: 'public' stamp: 'JanVanDeSandt 12/7/2012 21:13'!
handleAuthenticationCallback: aParameterDictionary 

	| code tokenRequest tokenResponse tokenData |
	
	code := aParameterDictionary at: 'code' ifAbsent: [ ^nil ].

	tokenRequest  := (ZnRequest post: (ZnUrl fromString: 'https://login.live.com/oauth20_token.srf'))
		entity: (ZnApplicationFormUrlEncodedEntity new
			at: 'code' put: code ;
			at: 'client_id' put: self consumerData key ;
			at: 'client_secret' put: self consumerData secret ;
			at: 'redirect_uri' put: self redirectUrl ;
			at: 'grant_type' put: 'authorization_code' ;
			yourself) ;
		yourself.

	tokenResponse := ZnClient new 
		request: tokenRequest ;
		post.
	tokenData := NeoJSONReader fromString: tokenResponse.
		
	accessToken := tokenData at: 'access_token'! !

!ZnOAuth2Session class methodsFor: 'instance creation' stamp: 'JanVanDeSandt 12/7/2012 15:32'!
facebook: aConsumerData

	^ ZnFacebookOAuth2Session new
		setConsumerData: aConsumerData ;
		yourself! !

!ZnOAuth2Session class methodsFor: 'instance creation' stamp: 'JanVanDeSandt 12/7/2012 15:32'!
google: aConsumerData

	^ ZnGoogleOAuth2Session new
		setConsumerData: aConsumerData ;
		yourself! !

!ZnOAuth2Session class methodsFor: 'instance creation' stamp: 'JanVanDeSandt 12/7/2012 20:57'!
microsoft: aConsumerData

	^ ZnMicrosoftOAuth2Session new
		setConsumerData: aConsumerData ;
		yourself! !

!ZnOAuth2Session class methodsFor: 'resources' stamp: 'JanVanDeSandt 12/23/2012 23:42'!
ssoScopes
	"Answer a collection of scopes that can be used for Authentication. "

	^ #()! !

!ZnOAuth2Session methodsFor: 'accessing' stamp: 'JanVanDeSandt 10/7/2012 14:04'!
accessToken
	^ accessToken! !

!ZnOAuth2Session methodsFor: 'public' stamp: 'JanVanDeSandt 10/7/2012 15:32'!
authenticationUrl
	"Answer a URL to the login page of the Identity Provider"
	
	self subclassResponsibility 
	! !

!ZnOAuth2Session methodsFor: 'accessing' stamp: 'JanVanDeSandt 10/7/2012 14:04'!
consumerData
	^ consumerData! !

!ZnOAuth2Session methodsFor: 'private' stamp: 'JanVanDeSandt 10/7/2012 14:17'!
fieldDictionaryFrom: aString

	| dict stream |
	
	dict := Dictionary new.
	stream := aString readStream.
	[ stream atEnd ] whileFalse: [ dict at: (stream upTo: $=) put: (stream upTo: $&) ].

	^dict! !

!ZnOAuth2Session methodsFor: 'public' stamp: 'JanVanDeSandt 12/7/2012 14:38'!
handleAuthenticationCallback: aParameterDictionary 

	self subclassResponsibility ! !

!ZnOAuth2Session methodsFor: 'initialize-release' stamp: 'JanVanDeSandt 12/23/2012 23:40'!
initialize

	super initialize.
	
	scopes := self class ssoScopes ! !

!ZnOAuth2Session methodsFor: 'accessing' stamp: 'JanVanDeSandt 12/7/2012 15:34'!
redirectUrl
	^ redirectUrl! !

!ZnOAuth2Session methodsFor: 'accessing' stamp: 'JanVanDeSandt 12/7/2012 15:34'!
redirectUrl: anObject
	redirectUrl := anObject! !

!ZnOAuth2Session methodsFor: 'private' stamp: 'JanVanDeSandt 12/23/2012 23:32'!
scopeParameter

	^ String streamContents: [ :stream |
		self scopes
			do: [ :each | stream nextPutAll: each ]
			separatedBy: [ stream nextPut: Character space ] ]! !

!ZnOAuth2Session methodsFor: 'accessing' stamp: 'JanVanDeSandt 12/23/2012 23:32'!
scopes

	^ scopes ifNil: [ scopes := OrderedCollection new ]! !

!ZnOAuth2Session methodsFor: 'accessing' stamp: 'JanVanDeSandt 12/23/2012 23:30'!
scopes: aCollection

	scopes := aCollection! !

!ZnOAuth2Session methodsFor: 'initialize-release' stamp: 'JanVanDeSandt 12/7/2012 15:34'!
setConsumerData: aConsumerData

	consumerData := aConsumerData.
	redirectUrl := aConsumerData redirectUrl! !

ZnDefaultServerDelegate subclass: #ZnSSOServerDelegate
	instanceVariableNames: 'facebookData googleData microsoftData twitterData'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-SSO-OAuth2-Core'!

!ZnSSOServerDelegate commentStamp: '<historical>' prior: 0!
I am ZnSSOServerDelegate, an example/test using SSO.

(ZnServer startOn: 8100)
	delegate: (ZnSSOServerDelegate new
		facebookData: (ZnOAuth2ConsumerData 
			key: '538399976180189'
			secret: '***************'
			redirectUrl: 'http://localhost:8100/sso-facebook-callback');
		googleData: (ZnOAuth2ConsumerData 
			key: '81499955779.apps.googleusercontent.com'
			secret: '***************'
			redirectUrl: 'http://localhost:8100/sso-google-callback');
		twitterData: (ZnOAuth1ConsumerData newForTwitterAuthentication 
			consumer: '5PAXu9CygKPD0aKwjnGdOA' ;
			consumerSecret: '*********');
		yourself);
	yourself.
!

!ZnSSOServerDelegate methodsFor: 'accessing' stamp: 'JanVanDeSandt 10/7/2012 13:32'!
facebookData
	^ facebookData! !

!ZnSSOServerDelegate methodsFor: 'accessing' stamp: 'JanVanDeSandt 10/7/2012 13:32'!
facebookData: anObject
	facebookData := anObject! !

!ZnSSOServerDelegate methodsFor: 'accessing-session' stamp: 'SvenVanCaekenberghe 12/20/2012 09:50'!
facebookSessionFor: aRequest
	^ aRequest session 
		attributeAt: 'facebook-session' 
		ifAbsentPut: [ ZnOAuth2Session facebook: self facebookData ]! !

!ZnSSOServerDelegate methodsFor: 'private' stamp: 'JanVanDeSandt 12/7/2012 11:14'!
generateInvalidCallbackRequest

	| html entity |

	html := String streamContents: [ :stream |
		stream
			nextPutAll: '<h3>Not a valid callback request</h3>' ].

	entity := ZnEntity html: html. 
	^ ZnResponse ok: entity! !

!ZnSSOServerDelegate methodsFor: 'accessing' stamp: 'JanVanDeSandt 10/7/2012 13:32'!
googleData
	^ googleData! !

!ZnSSOServerDelegate methodsFor: 'accessing' stamp: 'JanVanDeSandt 10/7/2012 13:32'!
googleData: anObject
	googleData := anObject! !

!ZnSSOServerDelegate methodsFor: 'accessing-session' stamp: 'SvenVanCaekenberghe 12/20/2012 09:49'!
googleSessionFor: aRequest
	^ aRequest session 
		attributeAt: 'google-session' 
		ifAbsentPut: [ ZnOAuth2Session google: self googleData ]! !

!ZnSSOServerDelegate methodsFor: 'private' stamp: 'SvenVanCaekenberghe 12/22/2012 19:01'!
htmlPageTitled: title do: block
	^ String streamContents: [ :stream |
		stream 
			<< '<?xml version="1.0" encoding="UTF-8"?>
<!!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html>
<head>
<title>';
			<< title;
			<< '</title>
<style type="text/css">
body {
  color: black;
  background: white;
  width: 900px;
  font-family: Verdana, Arial, Helvetica, sans-serif;
  font-size: 13px;
}
p {
  width: 600px;
  padding: 0 20px 10px 20px;
} 
ul,ol {
  width: 600px;
  padding: 0 5px 5px 30px;
} 
#logo {
  color: orange;
  font-family: Helvetica, sans-serif;
  font-weight: bold;
  font-size: 100px;
}
</style>
</head>
<body>
<div id="logo">Zn</div>
<h1>'; 
			<< title; 
			<<'</h1>'. 
		block value: stream. 
		stream << '</body></html>' ]! !

!ZnSSOServerDelegate methodsFor: 'initialize-release' stamp: 'JanVanDeSandt 12/24/2012 11:11'!
initialize

	super initialize.
	
	self prefixMap
		at: 'sso-demo' put: #ssoDemo: ;
		at: 'sso-reset' put: #ssoReset: ;
		at: 'sso-google' put: #ssoGoogle: ;
		at: 'sso-google-callback' put: #ssoGoogleCallback: ;
		at: 'sso-google-data' put: #ssoGoogleData: ;
		at: 'sso-facebook' put: #ssoFacebook: ;
		at: 'sso-facebook-callback' put: #ssoFacebookCallback: ;
		at: 'sso-facebook-data' put: #ssoFacebookData: ;
		at: 'sso-microsoft' put: #ssoMicrosoft: ;
		at: 'sso-microsoft-callback' put: #ssoMicrosoftCallback: ;
		at: 'sso-microsoft-data' put: #ssoMicrosoftData: ;
		at: 'sso-twitter' put: #ssoTwitter: ;
		at: 'sso-twitter-callback' put: #ssoTwitterCallback: ;
		at: 'sso-twitter-data' put: #ssoTwitterData:
! !

!ZnSSOServerDelegate methodsFor: 'testing' stamp: 'JanVanDeSandt 12/7/2012 11:10'!
isOAuth2Callback: aRequest

	^aRequest uri queryKeys includes: 'code'! !

!ZnSSOServerDelegate methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/20/2012 23:21'!
microsoftData
	^ microsoftData! !

!ZnSSOServerDelegate methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/20/2012 23:21'!
microsoftData: anObject
	microsoftData := anObject! !

!ZnSSOServerDelegate methodsFor: 'accessing-session' stamp: 'SvenVanCaekenberghe 12/20/2012 23:34'!
microsoftSessionFor: aRequest
	^ aRequest session 
		attributeAt: 'microsoft-session' 
		ifAbsentPut: [ ZnOAuth2Session microsoft: self microsoftData ]! !

!ZnSSOServerDelegate methodsFor: 'private' stamp: 'JanVanDeSandt 10/7/2012 14:38'!
printDictionary: aDictionary asHtmlOn: aStream

	aStream
		nextPutAll: '<ul>'.
	aDictionary keysAndValuesDo: [ :key :value |
		aStream
			nextPutAll: '<li>' ;
			nextPutAll: key.
		value isDictionary
			ifTrue: [ self printDictionary: value asHtmlOn: aStream ]
			ifFalse: [
				aStream
					nextPutAll: ' --> ' ;
					nextPutAll: value asString ].
		aStream nextPutAll: '</li>' ].
	aStream nextPutAll: '</ul>'! !

!ZnSSOServerDelegate methodsFor: 'accessing-session' stamp: 'JanVanDeSandt 12/24/2012 11:30'!
resetSessions: request
	#(#google #microsoft #facebook #twitter) do: [ :each | 
			request session removeAttribute: each , '-session' ]! !

!ZnSSOServerDelegate methodsFor: 'responses' stamp: 'SvenVanCaekenberghe 12/22/2012 19:19'!
ssoDemo: request
	| html |
	html := self
		htmlPageTitled: 'Zinc SSO Demo'
		do: [ :stream | 
			#(#Google #Facebook #Microsoft)
				do: [ :each | | session |
					session := self perform: (each asLowercase, 'SessionFor:') asSymbol with: request.
					stream
						<< '<a href="';
						<< session authenticationUrl asString;
						<< '" title="Login with ';
						<< each;
						<< '">';
						<< each;
						<< '</a>' ]
				separatedBy: [ stream << ' | ' ].
			stream << '</p><h4>May the Source be with you!!</h4>' ].
	^ ZnResponse ok: (ZnEntity html: html)! !

!ZnSSOServerDelegate methodsFor: 'responses' stamp: 'SvenVanCaekenberghe 12/22/2012 18:54'!
ssoFacebook: request

	| facebookSession html entity |

	facebookSession := self facebookSessionFor: request.
		
	html := self htmlPageTitled: 'Zinc SSO Facebook Login' do:  [ :stream |
		stream
			<< '<h3>Facebook SSO</h3>' ;
			<< '<a href="' ;
			<< facebookSession authenticationUrl asString ;
			<< '">Login with Facebook</a>' ].

	entity := ZnEntity html: html. 
	^ ZnResponse ok: entity! !

!ZnSSOServerDelegate methodsFor: 'responses' stamp: 'SvenVanCaekenberghe 12/23/2012 19:26'!
ssoFacebookCallback: request

	| facebookSession |
	
	(self isOAuth2Callback: request)
		ifFalse: [ ^ self generateInvalidCallbackRequest ].
	
	facebookSession := self facebookSessionFor: request.
	facebookSession handleAuthenticationCallback: request uri query.
	
	^ ZnResponse redirect: 'sso-facebook-data'! !

!ZnSSOServerDelegate methodsFor: 'responses' stamp: 'SvenVanCaekenberghe 12/23/2012 19:24'!
ssoFacebookData: request

	| facebookSession userData html entity |
	
	facebookSession := self facebookSessionFor: request.
	userData := facebookSession getUserData.

	html := self htmlPageTitled: 'Zinc SSO Facebook UserData' do: [ :stream | 
		stream << '<h3>Facebook UserData</h3>'.
		self printDictionary: userData asHtmlOn: stream.
		stream << '<a href="sso-reset">Reset</a>' ].

	entity := ZnEntity html: html. 
	^ ZnResponse ok: entity! !

!ZnSSOServerDelegate methodsFor: 'responses' stamp: 'SvenVanCaekenberghe 12/22/2012 18:57'!
ssoGoogle: request

	| googleSession html entity |
	
	googleSession := self googleSessionFor: request.
		
	html := self htmlPageTitled: 'Zinc SSO Google Login' do: [ :stream |
		stream
			<< '<h3>Google SSO</h3>' ;
			<< '<a href="' ;
			<< googleSession authenticationUrl asString ;
			<< '">Login with Google</a>' ].

	entity := ZnEntity html: html. 
	^ ZnResponse ok: entity! !

!ZnSSOServerDelegate methodsFor: 'responses' stamp: 'SvenVanCaekenberghe 12/23/2012 19:28'!
ssoGoogleCallback: request

	| googleSession |
	
	(self isOAuth2Callback: request)
		ifFalse: [ ^ self generateInvalidCallbackRequest ].

	googleSession := self googleSessionFor: request.
	googleSession handleAuthenticationCallback: request uri query.

	^ ZnResponse redirect: 'sso-google-data'! !

!ZnSSOServerDelegate methodsFor: 'responses' stamp: 'SvenVanCaekenberghe 12/23/2012 19:27'!
ssoGoogleData: request

	| googleSession userData html entity |
	
	googleSession := self googleSessionFor: request.
	userData := googleSession getUserData.

	html := self htmlPageTitled: 'Zinc SSO Google UserData' do: [ :stream | 
		stream << '<h3>Google UserData</h3>'.
		self printDictionary: userData asHtmlOn: stream.
		stream << '<a href="sso-reset">Reset</a>'  ].

	entity := ZnEntity html: html. 
	^ ZnResponse ok: entity! !

!ZnSSOServerDelegate methodsFor: 'responses' stamp: 'SvenVanCaekenberghe 12/22/2012 18:58'!
ssoMicrosoft: request

	| microsoftSession html entity |

	microsoftSession := self microsoftSessionFor: request.
		
	html := self htmlPageTitled: 'Zinc SSO Microsoft Login' do: [ :stream |
		stream
			<< '<h3>Microsoft SSO</h3>' ;
			<< '<a href="' ;
			<< microsoftSession authenticationUrl asString ;
			<< '">Login with Microsoft</a>' ].

	entity := ZnEntity html: html. 
	^ ZnResponse ok: entity! !

!ZnSSOServerDelegate methodsFor: 'responses' stamp: 'SvenVanCaekenberghe 12/23/2012 19:28'!
ssoMicrosoftCallback: request

	| microsoftSession |
	
	(self isOAuth2Callback: request)
		ifFalse: [ ^ self generateInvalidCallbackRequest ].

	microsoftSession := self microsoftSessionFor: request.
	microsoftSession handleAuthenticationCallback: request uri query.

	^ ZnResponse redirect: 'sso-microsoft-data'! !

!ZnSSOServerDelegate methodsFor: 'responses' stamp: 'SvenVanCaekenberghe 12/23/2012 19:27'!
ssoMicrosoftData: request

	| microsoftSession userData html entity |
	
	microsoftSession := self microsoftSessionFor: request.
	userData := microsoftSession getUserData.

	html := self htmlPageTitled: 'Zinc SSO Microsoft UserData' do: [ :stream | 
		stream << '<h3>Microsoft UserData</h3>'.
		self printDictionary: userData asHtmlOn: stream .
		stream << '<a href="sso-reset">Reset</a>' ].

	entity := ZnEntity html: html. 
	^ ZnResponse ok: entity! !

!ZnSSOServerDelegate methodsFor: 'responses' stamp: 'SvenVanCaekenberghe 12/22/2012 19:19'!
ssoReset: request
	| target |
	self resetSessions: request.
	target := request uri queryAt: #target ifAbsent: [ 'sso-demo' ].
	^ ZnResponse redirect: target! !

!ZnSSOServerDelegate methodsFor: 'responses' stamp: 'JanVanDeSandt 12/24/2012 11:15'!
ssoTwitter: request

	| oauthService requestToken html entity |

	oauthService := self twitterOAuth1Service.
	
	requestToken := oauthService getRequestTokenFor: 'http://localhost:8100/sso-twitter-callback'.
	self twitterSessionFor: request put: requestToken.
	
	html := self htmlPageTitled: 'Zinc SSO Twitter Login' do: [ :stream |
		stream
			<< '<h3>Twitter SSO</h3>' ;
			<< '<a href="' ;
			<< (oauthService loginUrlFor: requestToken) asString ;
			<< '">Login with Twitter</a>' ].

	entity := ZnEntity html: html. 
	^ ZnResponse ok: entity! !

!ZnSSOServerDelegate methodsFor: 'responses' stamp: 'JanVanDeSandt 12/24/2012 11:39'!
ssoTwitterCallback: request

	| requestToken oauthToken oauthVerifier accessToken |

	requestToken := self twitterSessionFor: request 
		ifAbsent:  [ ^ self generateInvalidCallbackRequest ].

	oauthToken := request uri queryAt: 'oauth_token'.
	oauthVerifier := request uri queryAt: 'oauth_verifier'.
	
	(oauthToken isNil or: [ oauthVerifier isNil])
		ifTrue: [ ^ self generateInvalidCallbackRequest ].
		
	oauthToken = requestToken value
		ifFalse: [ ^ self generateInvalidCallbackRequest ].
	
	accessToken := self twitterOAuth1Service getAccessToken: requestToken verifier: oauthVerifier.
	self twitterSessionFor: request put: accessToken. 

	^ ZnResponse redirect: 'sso-twitter-data'! !

!ZnSSOServerDelegate methodsFor: 'responses' stamp: 'JanVanDeSandt 12/27/2012 11:45'!
ssoTwitterData: request

	|  accessToken userData html entity |
	
	accessToken := self twitterSessionFor: request
		ifAbsent: [ ^ self generateInvalidCallbackRequest ].
		
	userData := ZnOAuth1TwitterUserAccess new
		oauth1Service: self twitterOAuth1Service ;
		accessToken: accessToken ;
		accountVerifyCredentials.
	
	html := self htmlPageTitled: 'Zinc SSO Twitter UserData' do: [ :stream | 
		stream << '<h3>Twitter UserData</h3>'.
		self printDictionary: userData asHtmlOn: stream .
		stream << '<a href="sso-reset">Reset</a>' ].

	entity := ZnEntity html: html. 
	^ ZnResponse ok: entity! !

!ZnSSOServerDelegate methodsFor: 'accessing' stamp: 'JanVanDeSandt 12/24/2012 10:28'!
twitterData

	^ twitterData! !

!ZnSSOServerDelegate methodsFor: 'accessing' stamp: 'JanVanDeSandt 12/24/2012 10:55'!
twitterData: aConsumerData

	twitterData := aConsumerData! !

!ZnSSOServerDelegate methodsFor: 'accessing' stamp: 'JanVanDeSandt 12/24/2012 10:58'!
twitterOAuth1Service

	^ ZnOAuth1Service new
		providerAccount: self twitterData ;
		yourself! !

!ZnSSOServerDelegate methodsFor: 'accessing-session' stamp: 'JanVanDeSandt 12/24/2012 11:34'!
twitterSessionFor: aRequest ifAbsent: aBlock

	^ aRequest session attributeAt: 'twitter-session' ifAbsent: aBlock! !

!ZnSSOServerDelegate methodsFor: 'accessing-session' stamp: 'JanVanDeSandt 12/24/2012 11:42'!
twitterSessionFor: aRequest put: anObject

	^ aRequest session attributeAt: 'twitter-session' put: anObject! !
